@@ -33,8 +33,9 @@ from supybot.test import *
 import supybot.conf as conf
 import supybot.plugin as plugin
 import supybot.registry as registry
+from supybot.utils.minisix import u
 
-import plugin as Aka
+from . import plugin as Aka
 
 class FunctionsTest(SupyTestCase):
     def testFindBiggestDollar(self):
@@ -102,7 +103,7 @@ class AkaChannelTestCase(ChannelPluginTestCase):
         self.assertNotError('aka add spam "echo [echo $*]"')
         self.assertResponse('spam egg', 'egg')
         self.assertResponse('spam egg bacon', 'egg bacon')
-    
+
     def testChannel(self):
         self.assertNotError('aka add channel echo $channel')
         self.assertResponse('aka channel', self.channel)
@@ -147,8 +148,8 @@ class AkaChannelTestCase(ChannelPluginTestCase):
         self.assertResponse('egg', 'baz')
 
     def testComplicatedNames(self):
-        self.assertNotError(u'aka add café "echo coffee"')
-        self.assertResponse(u'café', 'coffee')
+        self.assertNotError(u('aka add café "echo coffee"'))
+        self.assertResponse(u('café'), 'coffee')
 
         self.assertNotError('aka add "foo bar" "echo spam"')
         self.assertResponse('foo bar', 'spam')
@@ -201,7 +202,7 @@ class AkaTestCase(PluginTestCase):
 
     def testAliasImport(self):
         self.assertNotError('alias add foo "echo bar"')
-        self.assertNotError(u'alias add baz "echo café"')
+        self.assertNotError(u('alias add baz "echo café"'))
         self.assertNotError('aka add qux "echo quux"')
         self.assertResponse('alias foo', 'bar')
         self.assertResponse('alias baz', 'café')
@@ -232,11 +233,26 @@ class AkaTestCase(PluginTestCase):
         self.assertNotError('aka add "foo bar" baz')
         self.assertRegexp('aka list', 'foo.*?bar \$\*.*?foo bar.*?baz \$\*')
 
+    def testListLockedUnlocked(self):
+        self.assertNotError('register tacocat hunter2')
+
+        self.assertNotError('aka add foo bar')
+        self.assertNotError('aka add abcd echo hi')
+        self.assertNotError('aka lock foo')
+        self.assertRegexp('aka list --locked', 'foo')
+        self.assertNotRegexp('aka list --locked', 'abcd')
+        self.assertNotRegexp('aka list --unlocked', 'foo')
+        self.assertRegexp('aka list --unlocked', 'abcd')
+        # Can't look up both.
+        self.assertError('aka list --locked --unlocked abcd')
+
     def testSearch(self):
         self.assertNotError('aka add foo bar')
         self.assertNotError('aka add "many words" "much command"')
         self.assertRegexp('aka search f', 'foo')
         self.assertError('aka search abcdefghijklmnop')
         self.assertRegexp('aka search many', 'many words')
+        # This should be case insensitive too.
+        self.assertRegexp('aka search MaNY', 'many words')
 
 # vim:set shiftwidth=4 softtabstop=4 expandtab textwidth=79:
