@@ -28,16 +28,18 @@
 # POSSIBILITY OF SUCH DAMAGE.
 ###
 
+from __future__ import unicode_literals
+
 import re
 import sys
 import codecs
 import string
 import random
-from cStringIO import StringIO
 
 import supybot.conf as conf
 import supybot.utils as utils
 from supybot.commands import *
+import supybot.utils.minisix as minisix
 import supybot.ircmsgs as ircmsgs
 import supybot.ircutils as ircutils
 import supybot.callbacks as callbacks
@@ -60,7 +62,7 @@ class Filter(callbacks.Plugin):
         self.outFilters = ircutils.IrcDict()
 
     def outFilter(self, irc, msg):
-        if msg.command == 'PRIVMSG':
+        if msg.command in ('PRIVMSG', 'NOTICE'):
             if msg.args[0] in self.outFilters:
                 if ircmsgs.isAction(msg):
                     s = ircmsgs.unAction(msg)
@@ -116,14 +118,16 @@ class Filter(callbacks.Plugin):
         irc.reply(self._hebrew_remover(text))
     hebrew = wrap(hebrew, ['text'])
 
+    def _squish(self, text):
+        return text.replace(' ', '')
+
     @internationalizeDocstring
     def squish(self, irc, msg, args, text):
         """<text>
 
         Removes all the spaces from <text>.
         """
-        text = ''.join(text.split())
-        irc.reply(text)
+        irc.reply(self._squish(text))
     squish = wrap(squish, ['text'])
 
     @internationalizeDocstring
@@ -146,8 +150,7 @@ class Filter(callbacks.Plugin):
         Returns the binary representation of <text>.
         """
         L = []
-        if sys.version_info[0] >= 3:
-            print(repr(text))
+        if minisix.PY3:
             if isinstance(text, str):
                 bytes_ = text.encode()
             else:
@@ -181,8 +184,9 @@ class Filter(callbacks.Plugin):
         Returns the character representation of binary <text>.
         Assumes ASCII, 8 digits per character.
         """
+        text = self._squish(text)  # Strip spaces.
         try:
-            L = [chr(int(text[i:(i+8)], 2)) for i in xrange(0, len(text), 8)]
+            L = [chr(int(text[i:(i+8)], 2)) for i in range(0, len(text), 8)]
             irc.reply(''.join(L))
         except ValueError:
             irc.errorInvalid('binary string', text)
@@ -222,7 +226,7 @@ class Filter(callbacks.Plugin):
         commonly used for text that simply needs to be hidden from inadvertent
         reading by roaming eyes, since it's easily reversible.
         """
-        if sys.version_info[0] < 3:
+        if minisix.PY2:
             text = text.decode('utf8')
         irc.reply(self._rot13_encoder(text)[0])
     rot13 = wrap(rot13, ['text'])
@@ -389,7 +393,12 @@ class Filter(callbacks.Plugin):
 
         Returns <text> with each character randomly colorized.
         """
+        if minisix.PY2:
+            text = text.decode('utf-8')
+        text = ircutils.stripColor(text)
         L = [self._color(c) for c in text]
+        if minisix.PY2:
+            L = [c.encode('utf-8') for c in L]
         irc.reply('%s%s' % (''.join(L), '\x03'))
     colorize = wrap(colorize, ['text'])
 
@@ -399,12 +408,13 @@ class Filter(callbacks.Plugin):
 
         Returns <text> colorized like a rainbow.
         """
-        if sys.version_info[0] < 3:
+        if minisix.PY2:
             text = text.decode('utf-8')
+        text = ircutils.stripColor(text)
         colors = utils.iter.cycle(['05', '04', '07', '08', '09', '03', '11',
                                    '10', '12', '02', '06', '13'])
         L = [self._color(c, fg=next(colors)) for c in text]
-        if sys.version_info[0] < 3:
+        if minisix.PY2:
             L = [c.encode('utf-8') for c in L]
         irc.reply(''.join(L) + '\x03')
     rainbow = wrap(rainbow, ['text'])
@@ -509,7 +519,7 @@ class Filter(callbacks.Plugin):
             r'\blike\b': 'liek',
             r'[^e]ing\b': 'eing',
             }
-        for (r, s) in alwaysInsertions.iteritems():
+        for (r, s) in alwaysInsertions.items():
             text = re.sub(r, s, text)
         randomInsertions = {
             r'i': 'ui',
@@ -527,7 +537,7 @@ class Filter(callbacks.Plugin):
             r'\btheir\b': 'there',
             r'[^e]y': 'ey',
             }
-        for (r, s) in randomInsertions.iteritems():
+        for (r, s) in randomInsertions.items():
             text = re.sub(r, randomlyReplace(s), text)
         text = re.sub(r'(\w)\'(\w)', quoteOrNothing, text)
         text = re.sub(r'\.(\s+|$)', randomExclaims, text)
@@ -551,7 +561,7 @@ class Filter(callbacks.Plugin):
         'v': _('vee'), 'w': _('double-you'), 'x': _('ecks'), 'y': _('why'),
         'z': _('zee')
     }
-    for (k, v) in _spellLetters.items():
+    for (k, v) in list(_spellLetters.items()):
         _spellLetters[k.upper()] = v
     _spellPunctuation = {
         '!': _('exclamation point'),
@@ -607,10 +617,10 @@ class Filter(callbacks.Plugin):
             d.update(self._spellPunctuation)
 # A bug in unicode on OSX prevents me from testing this.
 ##         dd = {}
-##         for (c, v) in d.iteritems():
+##         for (c, v) in d.items():
 ##             dd[ord(c)] = unicode(v + ' ')
 ##         irc.reply(unicode(text).translate(dd))
-        out = StringIO()
+        out = minisix.io.StringIO()
         write = out.write
         for c in text:
             try:
@@ -654,39 +664,39 @@ class Filter(callbacks.Plugin):
     # XXX suckiest: B,K,P,Q,T
     # alternatives: 3: U+2107
     _uniudMap = {
-    ' ': u' ',      '0': u'0',      '@': u'@',
-    '!': u'\u00a1', '1': u'1',      'A': u'\u2200',
-    '"': u'\u201e', '2': u'\u2681', 'B': u'q',
-    '#': u'#',      '3': u'\u0190', 'C': u'\u0186',
-    '$': u'$',      '4': u'\u2683', 'D': u'\u15e1',
-    '%': u'%',      '5': u'\u1515', 'E': u'\u018e',
-    '&': u'\u214b', '6': u'9',      'F': u'\u2132',
-    "'": u'\u0375', '7': u'L',      'G': u'\u2141',
-    '(': u')',      '8': u'8',      'H': u'H',
-    ')': u'(',      '9': u'6',      'I': u'I',
-    '*': u'*',      ':': u':',      'J': u'\u148b',
-    '+': u'+',      ';': u';',      'K': u'\u029e',
-    ',': u'\u2018', '<': u'>',      'L': u'\u2142',
-    '-': u'-',      '=': u'=',      'M': u'\u019c',
-    '.': u'\u02d9', '>': u'<',      'N': u'N',
-    '/': u'/',      '?': u'\u00bf', 'O': u'O',
+    ' ': ' ',      '0': '0',      '@': '@',
+    '!': '\u00a1', '1': '1',      'A': '\u2200',
+    '"': '\u201e', '2': '\u2681', 'B': 'q',
+    '#': '#',      '3': '\u0190', 'C': '\u0186',
+    '$': '$',      '4': '\u2683', 'D': '\u15e1',
+    '%': '%',      '5': '\u1515', 'E': '\u018e',
+    '&': '\u214b', '6': '9',      'F': '\u2132',
+    "'": '\u0375', '7': 'L',      'G': '\u2141',
+    '(': ')',      '8': '8',      'H': 'H',
+    ')': '(',      '9': '6',      'I': 'I',
+    '*': '*',      ':': ':',      'J': '\u148b',
+    '+': '+',      ';': ';',      'K': '\u029e',
+    ',': '\u2018', '<': '>',      'L': '\u2142',
+    '-': '-',      '=': '=',      'M': '\u019c',
+    '.': '\u02d9', '>': '<',      'N': 'N',
+    '/': '/',      '?': '\u00bf', 'O': 'O',
 
-    'P': u'd',      '`': u'\u02ce', 'p': u'd',
-    'Q': u'b',      'a': u'\u0250', 'q': u'b',
-    'R': u'\u1d1a', 'b': u'q',      'r': u'\u0279',
-    'S': u'S',      'c': u'\u0254', 's': u's',
-    'T': u'\u22a5', 'd': u'p',      't': u'\u0287',
-    'U': u'\u144e', 'e': u'\u01dd', 'u': u'n',
-    'V': u'\u039b', 'f': u'\u214e', 'v': u'\u028c',
-    'W': u'M',      'g': u'\u0253', 'w': u'\u028d',
-    'X': u'X',      'h': u'\u0265', 'x': u'x',
-    'Y': u'\u2144', 'i': u'\u1d09', 'y': u'\u028e',
-    'Z': u'Z',      'j': u'\u027f', 'z': u'z',
-    '[': u']',      'k': u'\u029e', '{': u'}',
-    '\\': u'\\',    'l': u'\u05df', '|': u'|',
-    ']': u'[',      'm': u'\u026f', '}': u'{',
-    '^': u'\u2335', 'n': u'u',      '~': u'~',
-    '_': u'\u203e', 'o': u'o',
+    'P': 'd',      '`': '\u02ce', 'p': 'd',
+    'Q': 'b',      'a': '\u0250', 'q': 'b',
+    'R': '\u1d1a', 'b': 'q',      'r': '\u0279',
+    'S': 'S',      'c': '\u0254', 's': 's',
+    'T': '\u22a5', 'd': 'p',      't': '\u0287',
+    'U': '\u144e', 'e': '\u01dd', '': 'n',
+    'V': '\u039b', 'f': '\u214e', 'v': '\u028c',
+    'W': 'M',      'g': '\u0253', 'w': '\u028d',
+    'X': 'X',      'h': '\u0265', 'x': 'x',
+    'Y': '\u2144', 'i': '\u1d09', 'y': '\u028e',
+    'Z': 'Z',      'j': '\u027f', 'z': 'z',
+    '[': ']',      'k': '\u029e', '{': '}',
+    '\\': '\\',    'l': '\u05df', '|': '|',
+    ']': '[',      'm': '\u026f', '}': '{',
+    '^': '\u2335', 'n': '',      '~': '~',
+    '_': '\u203e', 'o': 'o',
     }
 
     @internationalizeDocstring
@@ -702,7 +712,7 @@ class Filter(callbacks.Plugin):
             if c in self._uniudMap:
                 tmp = self._uniudMap[c]
                 if not len(tmp):
-                    tmp = u'\ufffd'
+                    tmp = '\ufffd'
                 turned.append(tmp)
                 tlen += 1
             elif c == '\t':
@@ -712,7 +722,7 @@ class Filter(callbacks.Plugin):
             elif ord(c) >= 32:
                 turned.append(c)
                 tlen += 1
-        s = u'%s \x02 \x02' % u''.join(reversed(turned))
+        s = '%s \x02 \x02' % ''.join(reversed(turned))
         irc.reply(s)
     uniud = wrap(uniud, ['text'])
 Filter = internationalizeDocstring(Filter)
