@@ -32,6 +32,7 @@ import re
 import sys
 import time
 import uuid
+import functools
 
 import supybot.gpg as gpg
 import supybot.conf as conf
@@ -50,10 +51,30 @@ except ImportError:
     # without the i18n module
     _ = lambda x: x
 
+def check_gpg_available(f):
+    if gpg.available:
+        return f
+    else:
+        if not gpg.found_gnupg_lib:
+            def newf(self, irc, *args):
+                irc.error(_('gnupg features are not available because '
+                    'the python-gnupg library is not installed.'))
+        elif not gpg.found_gnupg_bin:
+            def newf(self, irc, *args):
+                irc.error(_('gnupg features are not available because '
+                    'the gnupg executable is not installed.'))
+        else:
+            # This case should never happen.
+            def newf(self, irc, *args):
+                irc.error(_('gnupg features are not available.'))
+        newf.__doc__ = f.__doc__
+        newf.__name__ = f.__name__
+        return newf
 
 class GPG(callbacks.Plugin):
     """Provides authentication based on GPG keys."""
     class key(callbacks.Commands):
+        @check_gpg_available
         def add(self, irc, msg, args, user, keyid, keyserver):
             """<key id> <key server>
 
@@ -79,6 +100,7 @@ class GPG(callbacks.Plugin):
                          ('somethingWithoutSpaces',
                              _('You must give a valid key server'))])
 
+        @check_gpg_available
         def remove(self, irc, msg, args, user, fingerprint):
             """<fingerprint>
 
@@ -99,6 +121,7 @@ class GPG(callbacks.Plugin):
                 irc.error(_('GPG key not associated with your account.'))
         remove = wrap(remove, ['user', 'somethingWithoutSpaces'])
 
+        @check_gpg_available
         def list(self, irc, msg, args, user):
             """takes no arguments
 
@@ -120,6 +143,7 @@ class GPG(callbacks.Plugin):
             self._tokens = dict(filter(lambda x_y: x_y[1][1]>now,
                 self._tokens.items()))
 
+        @check_gpg_available
         def gettoken(self, irc, msg, args):
             """takes no arguments
 
@@ -141,6 +165,7 @@ class GPG(callbacks.Plugin):
                 r'\r?\n-----END PGP SIGNATURE-----',
                 re.S)
         
+        @check_gpg_available
         def auth(self, irc, msg, args, url):
             """<url>
 
