@@ -30,10 +30,7 @@
 
 import re
 import sys
-import cgi
 import json
-import time
-import socket
 
 import supybot.conf as conf
 import supybot.utils as utils
@@ -46,8 +43,6 @@ import supybot.callbacks as callbacks
 from supybot.i18n import PluginInternationalization, internationalizeDocstring
 _ = PluginInternationalization('Google')
 
-import json
-
 class Google(callbacks.PluginRegexp):
     """This is a simple plugin to provide access to the Google services we
     all know and love from our favorite IRC bot."""
@@ -295,7 +290,9 @@ class Google(callbacks.PluginRegexp):
         """<source language> [to] <target language> <text>
 
         Returns <text> translated from <source language> into <target
-        language>.
+        language>. <source language> and <target language> take language
+        codes (not language names), which are listed here:
+        https://cloud.google.com/translate/v2/translate-reference#supported_languages
         """
         channel = msg.args[0]
         (text, language) = self._translate(sourceLang, targetLang, text)
