@@ -35,6 +35,7 @@ import csv
 import supybot.conf as conf
 import supybot.utils as utils
 from supybot.commands import *
+import supybot.utils.minisix as minisix
 import supybot.plugins as plugins
 import supybot.ircmsgs as ircmsgs
 import supybot.ircutils as ircutils
@@ -50,7 +51,7 @@ class SqliteKarmaDB(object):
         self.filename = filename
 
     def close(self):
-        for db in self.dbs.itervalues():
+        for db in self.dbs.values():
             db.close()
 
     def _getDb(self, channel):
@@ -59,12 +60,12 @@ class SqliteKarmaDB(object):
             return self.dbs[filename]
         if os.path.exists(filename):
             db = sqlite3.connect(filename, check_same_thread=False)
-            if sys.version_info[0] < 3:
+            if minisix.PY2:
                 db.text_factory = str
             self.dbs[filename] = db
             return db
         db = sqlite3.connect(filename, check_same_thread=False)
-        if sys.version_info[0] < 3:
+        if minisix.PY2:
             db.text_factory = str
         self.dbs[filename] = db
         cursor = db.cursor()
@@ -100,11 +101,11 @@ class SqliteKarmaDB(object):
         criteria = ' OR '.join(['normalized=?'] * len(normalizedThings))
         sql = """SELECT name, added-subtracted FROM karma
                  WHERE %s ORDER BY added-subtracted DESC""" % criteria
-        cursor.execute(sql, normalizedThings.keys())
+        cursor.execute(sql, list(normalizedThings.keys()))
         L = [(name, int(karma)) for (name, karma) in cursor.fetchall()]
         for (name, _) in L:
             del normalizedThings[name.lower()]
-        neutrals = normalizedThings.values()
+        neutrals = list(normalizedThings.values())
         neutrals.sort()
         return (L, neutrals)
 
