@@ -38,12 +38,13 @@ import string
 
 import supybot.utils as utils
 from supybot.commands import *
+import supybot.utils.minisix as minisix
 import supybot.callbacks as callbacks
 from supybot.i18n import PluginInternationalization, internationalizeDocstring
 _ = PluginInternationalization('Math')
 
 try:
-    from local import convertcore
+    from .local import convertcore
 except ImportError:
     from .local import convertcore
 
@@ -94,7 +95,7 @@ class Math(callbacks.Plugin):
     def _convertBaseToBase(self, number, toBase, fromBase):
         """Convert a number from any base, 2 through 36, to any other
         base, 2 through 36. Returns a string."""
-        number = long(str(number), fromBase)
+        number = minisix.long(str(number), fromBase)
         if toBase == 10:
             return str(number)
         return self._convertDecimalToBase(number, toBase)
@@ -109,13 +110,19 @@ class Math(callbacks.Plugin):
             return math.sqrt(x)
     def _cbrt(x):
         return math.pow(x, 1.0/3)
+    def _factorial(x):
+        if x<=10000:
+            return math.factorial(x)
+        else:
+            raise Exception('factorial argument too large')
     _mathEnv['sqrt'] = _sqrt
     _mathEnv['cbrt'] = _cbrt
     _mathEnv['abs'] = abs
     _mathEnv['max'] = max
     _mathEnv['min'] = min
-    _mathSafeEnv = dict([(x,y) for x,y in _mathEnv.items()
-        if x not in ['factorial']])
+    _mathEnv['round'] = round
+    _mathSafeEnv = dict([(x,y) for x,y in _mathEnv.items()])
+    _mathSafeEnv['factorial'] = _factorial
     _mathRe = re.compile(r'((?:(?<![A-Fa-f\d)])-)?'
                          r'(?:0x[A-Fa-f\d]+|'
                          r'0[0-7]+|'
@@ -182,6 +189,8 @@ class Math(callbacks.Plugin):
                             "Please remove them."))
             return
         if self._calc_match_forbidden_chars.match(text):
+            # Note: this is important to keep this to forbid usage of
+            # __builtins__
             irc.error(_('There\'s really no reason why you should have '
                            'underscores or brackets in your mathematical '
                            'expression.  Please remove them.'))
@@ -236,6 +245,8 @@ class Math(callbacks.Plugin):
         the 'trusted' capability to use.
         """
         if self._calc_match_forbidden_chars.match(text):
+            # Note: this is important to keep this to forbid usage of
+            # __builtins__
             irc.error(_('There\'s really no reason why you should have '
                            'underscores or brackets in your mathematical '
                            'expression.  Please remove them.'))
@@ -278,8 +289,8 @@ class Math(callbacks.Plugin):
                     x = abs(x)
                 stack.append(x)
             except ValueError: # Not a float.
-                if arg in self._mathEnv:
-                    f = self._mathEnv[arg]
+                if arg in self._mathSafeEnv:
+                    f = self._mathSafeEnv[arg]
                     if callable(f):
                         called = False
                         arguments = []
@@ -302,7 +313,7 @@ class Math(callbacks.Plugin):
                     arg1 = stack.pop()
                     s = '%s%s%s' % (arg1, arg, arg2)
                     try:
-                        stack.append(eval(s, self._mathEnv, self._mathEnv))
+                        stack.append(eval(s, self._mathSafeEnv, self._mathSafeEnv))
                     except SyntaxError:
                         irc.error(format(_('%q is not a defined function.'),
                                          arg))
