@@ -48,6 +48,7 @@ import supybot.irclib as irclib
 import supybot.plugin as plugin
 import supybot.plugins as plugins
 import supybot.drivers as drivers
+import supybot.utils.minisix as minisix
 import supybot.ircmsgs as ircmsgs
 import supybot.ircutils as ircutils
 import supybot.registry as registry
@@ -96,7 +97,7 @@ class Owner(callbacks.Plugin):
         # This needs to be done before we connect to any networks so that the
         # children of supybot.plugins (the actual plugins) exist and can be
         # loaded.
-        for (name, s) in registry._cache.iteritems():
+        for (name, s) in registry._cache.items():
             if 'alwaysLoadDefault' in name or 'alwaysLoadImportant' in name:
                 continue
             if name.startswith('supybot.plugins'):
@@ -156,7 +157,7 @@ class Owner(callbacks.Plugin):
             conf.supybot.networks.get(network).servers.append(serverS)
             assert conf.supybot.networks.get(network).servers(), \
                    'No servers are set for the %s network.' % network
-        self.log.info('Creating new Irc for %s.', network)
+        self.log.debug('Creating new Irc for %s.', network)
         newIrc = irclib.Irc(network)
         for irc in world.ircs:
             if irc != newIrc:
@@ -166,7 +167,7 @@ class Owner(callbacks.Plugin):
         return newIrc
 
     def _loadPlugins(self, irc):
-        self.log.info('Loading plugins (connecting to %s).', irc.network)
+        self.log.debug('Loading plugins (connecting to %s).', irc.network)
         alwaysLoadImportant = conf.supybot.plugins.alwaysLoadImportant()
         important = conf.supybot.commands.defaultPlugins.importantPlugins()
         for (name, value) in conf.supybot.plugins.getValues(fullNames=False):
@@ -246,15 +247,17 @@ class Owner(callbacks.Plugin):
                and self.commands.len(msg) > maximum \
                and not ircdb.checkCapability(msg.prefix, 'trusted'):
                 punishment = conf.supybot.abuse.flood.command.punishment()
-                banmask = ircutils.banmask(msg.prefix)
+                banmask = conf.supybot.protocols.irc.banmask \
+                        .makeBanmask(msg.prefix)
                 self.log.info('Ignoring %s for %s seconds due to an apparent '
                               'command flood.', banmask, punishment)
                 ircdb.ignores.add(banmask, time.time() + punishment)
-                irc.reply('You\'ve given me %s commands within the last '
-                          '%i seconds; I\'m now ignoring you for %s.' %
-                          (maximum,
-                           conf.supybot.abuse.flood.interval(),
-                           utils.timeElapsed(punishment, seconds=False)))
+                if conf.supybot.abuse.flood.command.notify():
+                    irc.reply('You\'ve given me %s commands within the last '
+                              '%i seconds; I\'m now ignoring you for %s.' %
+                              (maximum,
+                               conf.supybot.abuse.flood.interval(),
+                               utils.timeElapsed(punishment, seconds=False)))
                 return
             try:
                 tokens = callbacks.tokenize(s, channel=msg.args[0])
@@ -398,7 +401,8 @@ class Owner(callbacks.Plugin):
             L.append(format('linecache line cache flushed: %n cleared.',
                             (len(linecache.cache), 'line')))
             linecache.clearcache()
-            sys.exc_clear()
+            if minisix.PY2:
+                sys.exc_clear()
         collected = world.upkeep()
         if gc.garbage:
             L.append('Garbage!  %r.' % gc.garbage)
@@ -463,6 +467,7 @@ class Owner(callbacks.Plugin):
                 if hasattr(module, 'reload') and 'x' in locals():
                     module.reload(x)
                 if hasattr(module, 'config'):
+                    from imp import reload
                     reload(module.config)
                 for callback in callbacks:
                     callback.die()
