@@ -32,6 +32,7 @@ import gc
 import os
 import csv
 import time
+import codecs
 import fnmatch
 import os.path
 import threading
@@ -128,7 +129,7 @@ class ChannelDBHandler(object):
         return db
 
     def die(self):
-        for db in self.dbCache.itervalues():
+        for db in self.dbCache.values():
             try:
                 db.commit()
             except AttributeError: # In case it's not an SQLite database.
@@ -158,11 +159,11 @@ class DbiChannelDB(object):
         return db
 
     def close(self):
-        for db in self.dbs.itervalues():
+        for db in self.dbs.values():
             db.close()
 
     def flush(self):
-        for db in self.dbs.itervalues():
+        for db in self.dbs.values():
             db.flush()
 
     def __getattr__(self, attr):
@@ -200,14 +201,14 @@ class ChannelUserDictionary(collections.MutableMapping):
     def __len__(self):
         return sum([len(x) for x in self.channels])
 
-    def iteritems(self):
-        for (channel, ids) in self.channels.iteritems():
-            for (id, v) in ids.iteritems():
+    def items(self):
+        for (channel, ids) in self.channels.items():
+            for (id, v) in ids.items():
                 yield ((channel, id), v)
 
     def keys(self):
         L = []
-        for (k, _) in self.iteritems():
+        for (k, _) in self.items():
             L.append(k)
         return L
 
@@ -221,7 +222,7 @@ class ChannelUserDB(ChannelUserDictionary):
         ChannelUserDictionary.__init__(self)
         self.filename = filename
         try:
-            fd = open(self.filename)
+            fd = codecs.open(self.filename, encoding='utf8')
         except EnvironmentError as e:
             log.warning('Couldn\'t open %s: %s.', self.filename, e)
             return
@@ -250,9 +251,10 @@ class ChannelUserDB(ChannelUserDictionary):
             log.debug('Exception: %s', utils.exnToString(e))
 
     def flush(self):
-        fd = utils.file.AtomicFile(self.filename, makeBackupIfSmaller=False)
+        mode = 'wb' if utils.minisix.PY2 else 'w'
+        fd = utils.file.AtomicFile(self.filename, mode, makeBackupIfSmaller=False)
         writer = csv.writer(fd)
-        items = self.items()
+        items = list(self.items())
         if not items:
             log.debug('%s: Refusing to write blank file.',
                       self.__class__.__name__)
@@ -330,7 +332,9 @@ class ChannelIdDatabasePlugin(callbacks.Plugin):
                   (self.name(), id, channel))
 
     def checkChangeAllowed(self, irc, msg, channel, user, record):
-        if user.id == record.by:
+        # Checks and returns True if either the user ID (integer)
+        # or the hostmask of the caller match.
+        if (hasattr(user, 'id') and user.id == record.by) or user == record.by:
             return True
         cap = ircdb.makeChannelCapability(channel, 'op')
         if ircdb.checkCapability(msg.prefix, cap):
@@ -341,27 +345,38 @@ class ChannelIdDatabasePlugin(callbacks.Plugin):
         """This should irc.error or raise an exception if text is invalid."""
         pass
 
-    def add(self, irc, msg, args, user, channel, text):
+    def getUserId(self, irc, prefix, channel=None):
+        try:
+            user = ircdb.users.getUser(prefix)
+            return user.id
+        except KeyError:
+            if conf.get(conf.supybot.databases.plugins.requireRegistration, channel):
+                irc.errorNotRegistered(Raise=True)
+            return
+
+    def add(self, irc, msg, args, channel, text):
         """[<channel>] <text>
 
         Adds <text> to the $type database for <channel>.
         <channel> is only necessary if the message isn't sent in the channel
         itself.
         """
+        user = self.getUserId(irc, msg.prefix, channel) or msg.prefix
         at = time.time()
         self.addValidator(irc, text)
         if text is not None:
-            id = self.db.add(channel, at, user.id, text)
+            id = self.db.add(channel, at, user, text)
             irc.replySuccess('%s #%s added.' % (self.name(), id))
-    add = wrap(add, ['user', 'channeldb', 'text'])
+    add = wrap(add, ['channeldb', 'text'])
 
-    def remove(self, irc, msg, args, user, channel, id):
+    def remove(self, irc, msg, args, channel, id):
         """[<channel>] <id>
 
         Removes the $type with id <id> from the $type database for <channel>.
         <channel> is only necessary if the message isn't sent in the channel
         itself.
         """
+        user = self.getUserId(irc, msg.prefix, channel) or msg.prefix
         try:
             record = self.db.get(channel, id)
             self.checkChangeAllowed(irc, msg, channel, user, record)
@@ -369,7 +384,7 @@ class ChannelIdDatabasePlugin(callbacks.Plugin):
             irc.replySuccess()
         except KeyError:
             self.noSuchRecord(irc, channel, id)
-    remove = wrap(remove, ['user', 'channeldb', 'id'])
+    remove = wrap(remove, ['channeldb', 'id'])
 
     def searchSerializeRecord(self, record):
         text = utils.str.ellipsisify(record.text, 50)
@@ -430,13 +445,14 @@ class ChannelIdDatabasePlugin(callbacks.Plugin):
             self.noSuchRecord(irc, channel, id)
     get = wrap(get, ['channeldb', 'id'])
 
-    def change(self, irc, msg, args, user, channel, id, replacer):
+    def change(self, irc, msg, args, channel, id, replacer):
         """[<channel>] <id> <regexp>
 
         Changes the $type with id <id> according to the regular expression
         <regexp>.  <channel> is only necessary if the message isn't sent in the
         channel itself.
         """
+        user = self.getUserId(irc, msg.prefix, channel) or msg.prefix
         try:
             record = self.db.get(channel, id)
             self.checkChangeAllowed(irc, msg, channel, user, record)
@@ -445,7 +461,7 @@ class ChannelIdDatabasePlugin(callbacks.Plugin):
             irc.replySuccess()
         except KeyError:
             self.noSuchRecord(irc, channel, id)
-    change = wrap(change, ['user', 'channeldb', 'id', 'regexpReplacer'])
+    change = wrap(change, ['channeldb', 'id', 'regexpReplacer'])
 
     def stats(self, irc, msg, args, channel):
         """[<channel>]
