@@ -34,9 +34,15 @@ import time
 import socket
 
 from . import ircutils, registry, utils
+from .utils import minisix
+from .utils.net import isSocketAddress
 from .version import version
 from .i18n import PluginInternationalization
 _ = PluginInternationalization()
+if minisix.PY2:
+    from urllib2 import build_opener, install_opener, ProxyHandler
+else:
+    from urllib.request import build_opener, install_opener, ProxyHandler
 
 ###
 # *** The following variables are affected by command-line options.  They are
@@ -210,7 +216,7 @@ class VersionIfEmpty(registry.String):
     def __call__(self):
         ret = registry.String.__call__(self)
         if not ret:
-            ret = 'Supybot %s' % version
+            ret = 'Limnoria %s' % version
         return ret
 
 registerGlobalValue(supybot, 'user',
@@ -300,23 +306,44 @@ class SpaceSeparatedSetOfChannels(registry.SpaceSeparatedListOf):
             # Let's be explicit about it
             return None
 
-def registerNetwork(name, password='', ssl=False, sasl_username='',
+class ValidSaslMechanism(registry.OnlySomeStrings):
+    validStrings = ('ecdsa-nist256p-challenge', 'external', 'plain')
+
+class SpaceSeparatedListOfSaslMechanisms(registry.SpaceSeparatedListOf):
+    Value = ValidSaslMechanism
+
+def registerNetwork(name, password='', ssl=True, sasl_username='',
         sasl_password=''):
     network = registerGroup(supybot.networks, name)
     registerGlobalValue(network, 'password', registry.String(password,
         _("""Determines what password will be used on %s.  Yes, we know that
         technically passwords are server-specific and not network-specific,
         but this is the best we can do right now.""") % name, private=True))
-    registryServers = registerGlobalValue(network, 'servers', Servers([],
+    registerGlobalValue(network, 'servers', Servers([],
         _("""Space-separated list of servers the bot will connect to for %s.
         Each will be tried in order, wrapping back to the first when the cycle
         is completed.""") % name))
     registerGlobalValue(network, 'channels', SpaceSeparatedSetOfChannels([],
         _("""Space-separated list of channels the bot will join only on %s.""")
         % name, private=True))
+
     registerGlobalValue(network, 'ssl', registry.Boolean(ssl,
         _("""Determines whether the bot will attempt to connect with SSL
         sockets to %s.""") % name))
+    registerGlobalValue(network.ssl, 'serverFingerprints',
+        registry.SpaceSeparatedSetOfStrings([], format(_("""Space-separated list
+        of fingerprints of trusted certificates for this network.
+        Supported hash algorithms are: %L.
+        If non-empty, Certification Authority signatures will not be used to
+        verify certificates."""), utils.net.FINGERPRINT_ALGORITHMS)))
+    registerGlobalValue(network.ssl, 'authorityCertificate',
+        registry.String('', _("""A certificate that is trusted to verify
+        certificates of this network (aka. Certificate Authority).""")))
+    registerGlobalValue(network, 'requireStarttls', registry.Boolean(False,
+        _("""Determines whether the bot will connect in plain text to %s
+        but require STARTTLS before authentication. This is ignored if the
+        connection already uses SSL.""") % name))
+
     registerGlobalValue(network, 'certfile', registry.String('',
         _("""Determines what certificate file (if any) the bot will use to
         connect with SSL sockets to %s.""") % name))
@@ -348,6 +375,9 @@ def registerNetwork(name, password='', ssl=False, sasl_username='',
         _("""Determines what SASL ECDSA key (if any) will be used on %s.
         The public key must be registered with NickServ for SASL
         ECDSA-NIST256P-CHALLENGE to work.""") % name, private=False))
+    registerGlobalValue(sasl, 'mechanisms', SpaceSeparatedListOfSaslMechanisms(
+        ['ecdsa-nist256p-challenge', 'external', 'plain'], _("""Determines
+        what SASL mechanisms will be tried and in which order.""")))
     registerGlobalValue(network, 'socksproxy', registry.String('',
         _("""If not empty, determines the hostname of the socks proxy that
         will be used to connect to this network.""")))
@@ -370,6 +400,12 @@ registerGroup(supybot, 'reply')
 registerGroup(supybot.reply, 'format')
 registerChannelValue(supybot.reply.format, 'url',
     registry.String('<%s>', _("""Determines how urls should be formatted.""")))
+def url(s):
+    if s:
+        return supybot.reply.format.url() % s
+    else:
+        return ''
+utils.str.url = url
 registerChannelValue(supybot.reply.format, 'time',
     registry.String('%Y-%m-%dT%H:%M:%S%z', _("""Determines how timestamps
     printed for human reading should be formatted. Refer to the Python
@@ -450,7 +486,8 @@ registerChannelValue(supybot.reply.error, 'withNotice',
     supybot.reply.errorInPrivate so private errors don't open a query window
     in most IRC clients.""")))
 registerChannelValue(supybot.reply.error, 'noCapability',
-    registry.Boolean(False, _("""Determines whether the bot will send an error
+    registry.Boolean(False, _("""Determines whether the bot will *not* provide
+    details in the error
     message to users who attempt to call a command for which they do not have
     the necessary capability.  You may wish to make this True if you don't want
     users to understand the underlying security system preventing them from
@@ -726,6 +763,9 @@ registerGlobalValue(supybot.abuse.flood.command, 'maximum',
 registerGlobalValue(supybot.abuse.flood.command, 'punishment',
     registry.PositiveInteger(300, _("""Determines how many seconds the bot
     will ignore users who flood it with commands.""")))
+registerGlobalValue(supybot.abuse.flood.command, 'notify',
+    registry.Boolean(True, _("""Determines whether the bot will notify people
+    that they're being ignored for command flooding.""")))
 
 registerGlobalValue(supybot.abuse.flood.command, 'invalid',
     registry.Boolean(True, _("""Determines whether the bot will defend itself
@@ -931,6 +971,11 @@ class ChannelSpecific(registry.Boolean):
         return lchannel
 
 registerGroup(supybot.databases, 'plugins')
+
+registerChannelValue(supybot.databases.plugins, 'requireRegistration',
+    registry.Boolean(True, _("""Determines whether the bot will require user
+    registration to use 'add' commands in database-based Supybot
+    plugins.""")))
 registerChannelValue(supybot.databases.plugins, 'channelSpecific',
     ChannelSpecific(True, _("""Determines whether database-based plugins that
     can be channel-specific will be so.  This can be overridden by individual
@@ -1125,16 +1170,42 @@ registerGlobalValue(supybot.protocols.irc.queuing.rateLimit, 'join',
 ###
 registerGroup(supybot.protocols, 'http')
 registerGlobalValue(supybot.protocols.http, 'peekSize',
-    registry.PositiveInteger(4096, _("""Determines how many bytes the bot will
+    registry.PositiveInteger(8192, _("""Determines how many bytes the bot will
     'peek' at when looking through a URL for a doctype or title or something
     similar.  It'll give up after it reads this many bytes, even if it hasn't
     found what it was looking for.""")))
 
+class HttpProxy(registry.String):
+    """Value must be a valid hostname:port string."""
+    def setValue(self, v):
+        proxies = {}
+        if v != "":
+            if isSocketAddress(v):
+                proxies = {
+                    'http': v,
+                    'https': v
+                    }
+            else:
+                self.error()
+        proxyHandler = ProxyHandler(proxies)
+        proxyOpenerDirector = build_opener(proxyHandler)
+        install_opener(proxyOpenerDirector)
+        super(HttpProxy, self).setValue(v)
+
 registerGlobalValue(supybot.protocols.http, 'proxy',
-    registry.String('', _("""Determines what proxy all HTTP requests should go
+    HttpProxy('', _("""Determines what proxy all HTTP requests should go
     through.  The value should be of the form 'host:port'.""")))
 utils.web.proxy = supybot.protocols.http.proxy
 
+###
+# supybot.protocols.ssl
+###
+registerGroup(supybot.protocols, 'ssl')
+registerGlobalValue(supybot.protocols.ssl, 'verifyCertificates',
+    registry.Boolean(False, _("""Determines whether server certificates
+    will be verified, which checks whether the server certificate is signed
+    by a known certificate authority, and aborts the connection if it is not.""")))
+
 
 ###
 # HTTP server
@@ -1150,8 +1221,12 @@ class IP(registry.String):
         else:
             registry.String.setValue(self, v)
 
+registerGlobalValue(supybot.servers.http, 'singleStack',
+    registry.Boolean(True, _("""If true, uses IPV6_V6ONLY to disable
+    forwaring of IPv4 traffic to IPv6 sockets. On *nix, has the same
+    effect as setting kernel variable net.ipv6.bindv6only to 1.""")))
 registerGlobalValue(supybot.servers.http, 'hosts4',
-    IP('', _("""Space-separated list of IPv4 hosts the HTTP server
+    IP('0.0.0.0', _("""Space-separated list of IPv4 hosts the HTTP server
     will bind.""")))
 registerGlobalValue(supybot.servers.http, 'hosts6',
     IP('::0', _("""Space-separated list of IPv6 hosts the HTTP server will
