@@ -32,6 +32,7 @@ import copy
 import time
 import random
 import base64
+import collections
 
 try:
     from ecdsa import SigningKey, BadDigestError
@@ -42,7 +43,7 @@ except ImportError:
 from . import conf, ircdb, ircmsgs, ircutils, log, utils, world
 from .utils.str import rsplit
 from .utils.iter import chain
-from .utils.structures import queue, smallqueue, RingBuffer
+from .utils.structures import smallqueue, RingBuffer
 
 ###
 # The base class for a callback to be registered with an Irc object.  Shows
@@ -60,7 +61,7 @@ class IrcCommandDispatcher(object):
         return getattr(self, 'do' + command.capitalize(), None)
 
 
-class IrcCallback(IrcCommandDispatcher):
+class IrcCallback(IrcCommandDispatcher, log.Firewalled):
     """Base class for standard callbacks.
 
     Callbacks derived from this class should have methods of the form
@@ -69,7 +70,6 @@ class IrcCallback(IrcCommandDispatcher):
     """
     callAfter = ()
     callBefore = ()
-    __metaclass__ = log.MetaFirewall
     __firewalled__ = {'die': None,
                       'reset': None,
                       '__call__': None,
@@ -342,14 +342,16 @@ class ChannelState(utils.python.Object):
             ret = ret and getattr(self, name) == getattr(other, name)
         return ret
 
+Batch = collections.namedtuple('Batch', 'type arguments messages')
 
-class IrcState(IrcCommandDispatcher):
+class IrcState(IrcCommandDispatcher, log.Firewalled):
     """Maintains state of the Irc connection.  Should also become smarter.
     """
-    __metaclass__ = log.MetaFirewall
     __firewalled__ = {'addMsg': None}
     def __init__(self, history=None, supported=None,
-                 nicksToHostmasks=None, channels=None):
+                 nicksToHostmasks=None, channels=None,
+                 capabilities_ack=None, capabilities_nak=None,
+                 capabilities_ls=None):
         if history is None:
             history = RingBuffer(conf.supybot.protocols.irc.maxHistoryLength())
         if supported is None:
@@ -358,11 +360,15 @@ class IrcState(IrcCommandDispatcher):
             nicksToHostmasks = ircutils.IrcDict()
         if channels is None:
             channels = ircutils.IrcDict()
+        self.capabilities_ack = capabilities_ack or set()
+        self.capabilities_nak = capabilities_nak or set()
+        self.capabilities_ls = capabilities_ls or {}
         self.ircd = None
         self.supported = supported
         self.history = history
         self.channels = channels
         self.nicksToHostmasks = nicksToHostmasks
+        self.batches = {}
 
     def reset(self):
         """Resets the state to normal, unconnected state."""
@@ -371,6 +377,7 @@ class IrcState(IrcCommandDispatcher):
         self.supported.clear()
         self.nicksToHostmasks.clear()
         self.history.resize(conf.supybot.protocols.irc.maxHistoryLength())
+        self.batches = {}
 
     def __reduce__(self):
         return (self.__class__, (self.history, self.supported,
@@ -380,7 +387,8 @@ class IrcState(IrcCommandDispatcher):
         return self.history == other.history and \
                self.channels == other.channels and \
                self.supported == other.supported and \
-               self.nicksToHostmasks == other.nicksToHostmasks
+               self.nicksToHostmasks == other.nicksToHostmasks and \
+               self.batches == other.batches
 
     def __ne__(self, other):
         return not self == other
@@ -390,6 +398,7 @@ class IrcState(IrcCommandDispatcher):
         ret.history = copy.deepcopy(self.history)
         ret.nicksToHostmasks = copy.deepcopy(self.nicksToHostmasks)
         ret.channels = copy.deepcopy(self.channels)
+        ret.batches = copy.deepcopy(self.batches)
         return ret
 
     def addMsg(self, irc, msg):
@@ -397,6 +406,11 @@ class IrcState(IrcCommandDispatcher):
         self.history.append(msg)
         if ircutils.isUserHostmask(msg.prefix) and not msg.command == 'NICK':
             self.nicksToHostmasks[msg.nick] = msg.prefix
+        if 'batch' in msg.server_tags:
+            batch = msg.server_tags['batch']
+            assert batch in self.batches, \
+                    'Server references undeclared batch %s' % batch
+            self.batches[batch].messages.append(msg)
         method = self.dispatchCommand(msg.command)
         if method is not None:
             method(irc, msg)
@@ -506,15 +520,26 @@ class IrcState(IrcCommandDispatcher):
 
     def do353(self, irc, msg):
         # NAMES reply.
-        (__, type, channel, names) = msg.args
+        (__, type, channel, items) = msg.args
         if channel not in self.channels:
             self.channels[channel] = ChannelState()
         c = self.channels[channel]
-        for name in names.split():
+        for item in items.split():
+            if ircutils.isUserHostmask(item):
+                name = ircutils.nickFromHostmask(item)
+                self.nicksToHostmasks[name] = name
+            else:
+                name = item
             c.addUser(name)
         if type == '@':
             c.modes['s'] = None
 
+    def doChghost(self, irc, msg):
+        (user, host) = msg.args
+        nick = msg.nick
+        hostmask = '%s!%s@%s' % (nick, user, host)
+        self.nicksToHostmasks[nick] = hostmask
+
     def doJoin(self, irc, msg):
         for channel in msg.args[0].split(','):
             if channel in self.channels:
@@ -594,8 +619,13 @@ class IrcState(IrcCommandDispatcher):
                 chan.removeUser(user)
 
     def doQuit(self, irc, msg):
-        for channel in self.channels.itervalues():
-            channel.removeUser(msg.nick)
+        channel_names = ircutils.IrcSet()
+        for (name, channel) in self.channels.items():
+            if msg.nick in channel.users:
+                channel_names.add(name)
+                channel.removeUser(msg.nick)
+        # Remember which channels the user was on
+        msg.tag('channels', channel_names)
         if msg.nick in self.nicksToHostmasks:
             # If we're quitting, it may not be.
             del self.nicksToHostmasks[msg.nick]
@@ -625,9 +655,32 @@ class IrcState(IrcCommandDispatcher):
             del self.nicksToHostmasks[oldNick]
         except KeyError:
             pass
-        for channel in self.channels.itervalues():
+        channel_names = ircutils.IrcSet()
+        for (name, channel) in self.channels.items():
+            if msg.nick in channel.users:
+                channel_names.add(name)
             channel.replaceUser(oldNick, newNick)
+        msg.tag('channels', channel_names)
 
+    def doBatch(self, irc, msg):
+        batch_name = msg.args[0][1:]
+        if msg.args[0].startswith('+'):
+            batch_type = msg.args[1]
+            batch_arguments = tuple(msg.args[2:])
+            self.batches[batch_name] = Batch(type=batch_type,
+                    arguments=batch_arguments, messages=[])
+        elif msg.args[0].startswith('-'):
+            batch = self.batches.pop(batch_name)
+            msg.tag('batch', batch)
+        else:
+            assert False, msg.args[0]
+
+    def doAway(self, irc, msg):
+        channel_names = ircutils.IrcSet()
+        for (name, channel) in self.channels.items():
+            if msg.nick in channel.users:
+                channel_names.add(name)
+        msg.tag('channels', channel_names)
 
 
 ###
@@ -636,12 +689,11 @@ class IrcState(IrcCommandDispatcher):
 # 'queue', and 'state', in addition to the standard nick/user/ident attributes.
 ###
 _callbacks = []
-class Irc(IrcCommandDispatcher):
+class Irc(IrcCommandDispatcher, log.Firewalled):
     """The base class for an IRC connection.
 
     Handles PING commands already.
     """
-    __metaclass__ = log.MetaFirewall
     __firewalled__ = {'die': None,
                       'feedMsg': None,
                       'takeMsg': None,}
@@ -663,6 +715,7 @@ class Irc(IrcCommandDispatcher):
         self._setNonResettingVariables()
         self._queueConnectMessages()
         self.startedSync = ircutils.IrcDict()
+        self.monitoring = ircutils.IrcDict()
 
     def isChannel(self, s):
         """Helper function to check whether a given string is a channel on
@@ -912,29 +965,15 @@ class Irc(IrcCommandDispatcher):
 
     def _setNonResettingVariables(self):
         # Configuration stuff.
+        network_config = conf.supybot.networks.get(self.network)
         def get_value(name):
-            return getattr(conf.supybot.networks.get(self.network), name)() or \
+            return getattr(network_config, name)() or \
                 getattr(conf.supybot, name)()
         self.nick = get_value('nick')
         self.user = get_value('user')
         self.ident = get_value('ident')
         self.alternateNicks = conf.supybot.nick.alternates()[:]
-        self.password = conf.supybot.networks.get(self.network).password()
-        self.sasl_username = \
-                conf.supybot.networks.get(self.network).sasl.username()
-        # TODO Find a better way to fix this
-        if hasattr(self.sasl_username, 'decode'):
-            self.sasl_username = self.sasl_username.decode('utf-8')
-        self.sasl_password = \
-                conf.supybot.networks.get(self.network).sasl.password()
-        # TODO Find a better way to fix this
-        if hasattr(self.sasl_password, 'decode'):
-            self.sasl_password = self.sasl_password.decode('utf-8')
-        self.sasl_ecdsa_key = \
-                conf.supybot.networks.get(self.network).sasl.ecdsa_key()
-        # TODO Find a better way to fix this
-        if hasattr(self.sasl_ecdsa_key, 'decode'):
-            self.sasl_ecdsa_key = self.sasl_ecdsa_key.decode('utf-8')
+        self.password = network_config.password()
         self.prefix = '%s!%s@%s' % (self.nick, self.ident, 'unset.domain')
         # The rest.
         self.lastTake = 0
@@ -943,7 +982,41 @@ class Irc(IrcCommandDispatcher):
         self.startedAt = time.time()
         self.lastping = time.time()
         self.outstandingPing = False
+        self.capNegociationEnded = False
+        self.requireStarttls = not network_config.ssl() and \
+                network_config.requireStarttls()
+        self.resetSasl()
 
+    def resetSasl(self):
+        network_config = conf.supybot.networks.get(self.network)
+        self.sasl_authenticated = False
+        self.sasl_username = network_config.sasl.username()
+        self.sasl_password = network_config.sasl.password()
+        self.sasl_ecdsa_key = network_config.sasl.ecdsa_key()
+        self.authenticate_decoder = None
+        self.sasl_next_mechanisms = []
+        self.sasl_current_mechanism = None
+
+        for mechanism in network_config.sasl.mechanisms():
+            if mechanism == 'ecdsa-nist256p-challenge' and \
+                    ecdsa and self.sasl_username and self.sasl_ecdsa_key:
+                self.sasl_next_mechanisms.append(mechanism)
+            elif mechanism == 'external' and (
+                    network_config.certfile() or
+                    conf.supybot.protocols.irc.certfile()):
+                self.sasl_next_mechanisms.append(mechanism)
+            elif mechanism == 'plain' and \
+                    self.sasl_username and self.sasl_password:
+                self.sasl_next_mechanisms.append(mechanism)
+
+        if self.sasl_next_mechanisms:
+            self.REQUEST_CAPABILITIES.add('sasl')
+
+
+    REQUEST_CAPABILITIES = set(['account-notify', 'extended-join',
+        'multi-prefix', 'metadata-notify', 'account-tag',
+        'userhost-in-names', 'invite-notify', 'server-time',
+        'chghost', 'batch', 'away-notify'])
 
     def _queueConnectMessages(self):
         if self.zombie:
@@ -952,124 +1025,280 @@ class Irc(IrcCommandDispatcher):
 
             return
 
-        self.sasl = None
+        self.sendMsg(ircmsgs.IrcMsg(command='CAP', args=('LS', '302')))
 
-        if ecdsa and self.sasl_username and self.sasl_ecdsa_key:
-            self.sasl = 'ecdsa-nist256p-challenge'
-        elif (conf.supybot.networks.get(self.network).certfile() or
-                conf.supybot.protocols.irc.certfile()):
-            self.sasl = 'external'
-        elif self.sasl_username and self.sasl_password:
-            self.sasl = 'plain'
-
-        for cap in ('account-notify', 'extended-join', 'multi-prefix'):
-            self.queueMsg(ircmsgs.IrcMsg(command='CAP', args=('REQ', cap)))
-
-        if self.sasl:
-            self.queueMsg(ircmsgs.IrcMsg(command='CAP', args=('REQ', 'sasl')))
+        if self.requireStarttls:
+            self.sendMsg(ircmsgs.IrcMsg(command='STARTTLS'))
         else:
-            self.queueMsg(ircmsgs.IrcMsg(command='CAP', args=('END',)))
+            self.sendAuthenticationMessages()
+
+    def do670(self, irc, msg):
+        """STARTTLS accepted."""
+        log.info('%s: Starting TLS session.', self.network)
+        self.requireStarttls = False
+        self.driver.starttls()
+        self.sendAuthenticationMessages()
+    def do691(self, irc, msg):
+        """STARTTLS refused."""
+        log.error('%s: Server refused STARTTLS: %s', self.network, msg.args[0])
+        self.feedMsg(ircmsgs.error('STARTTLS upgrade refused by the server'))
+        self.driver.reconnect()
+
+    def sendAuthenticationMessages(self):
+        # Notes:
+        # * using sendMsg instead of queueMsg because these messages cannot
+        #   be throttled.
 
         if self.password:
             log.info('%s: Queuing PASS command, not logging the password.',
                      self.network)
+            self.sendMsg(ircmsgs.password(self.password))
 
-            self.queueMsg(ircmsgs.password(self.password))
-
-        log.debug('%s: Queuing NICK command, nick is %s.',
+        log.debug('%s: Sending NICK command, nick is %s.',
                   self.network, self.nick)
 
-        self.queueMsg(ircmsgs.nick(self.nick))
+        self.sendMsg(ircmsgs.nick(self.nick))
 
-        log.debug('%s: Queuing USER command, ident is %s, user is %s.',
+        log.debug('%s: Sending USER command, ident is %s, user is %s.',
                   self.network, self.ident, self.user)
 
-        self.queueMsg(ircmsgs.user(self.ident, self.user))
+        self.sendMsg(ircmsgs.user(self.ident, self.user))
+
+    def endCapabilityNegociation(self):
+        if not self.capNegociationEnded:
+            self.capNegociationEnded = True
+            self.sendMsg(ircmsgs.IrcMsg(command='CAP', args=('END',)))
+
+    def sendSaslString(self, string):
+        for chunk in ircutils.authenticate_generator(string):
+            self.sendMsg(ircmsgs.IrcMsg(command='AUTHENTICATE',
+                args=(chunk,)))
+
+    def tryNextSaslMechanism(self):
+        if self.sasl_next_mechanisms:
+            self.sasl_current_mechanism = self.sasl_next_mechanisms.pop(0)
+            self.sendMsg(ircmsgs.IrcMsg(command='AUTHENTICATE',
+                args=(self.sasl_current_mechanism.upper(),)))
+        else:
+            self.sasl_current_mechanism = None
+            self.endCapabilityNegociation()
+
+    def filterSaslMechanisms(self, available):
+        available = set(map(str.lower, available))
+        self.sasl_next_mechanisms = [
+                x for x in self.sasl_next_mechanisms
+                if x.lower() in available]
 
     def doAuthenticate(self, msg):
-        if len(msg.args) == 1 and msg.args[0] == '+':
-            log.info('%s: Authenticating using SASL.', self.network)
+        if not self.authenticate_decoder:
+            self.authenticate_decoder = ircutils.AuthenticateDecoder()
+        self.authenticate_decoder.feed(msg)
+        if not self.authenticate_decoder.ready:
+            return # Waiting for other messages
+        string = self.authenticate_decoder.get()
+        self.authenticate_decoder = None
 
-            if self.sasl == 'external':
-                authstring = '+'
-            elif self.sasl == 'ecdsa-nist256p-challenge':
-                authstring = base64.b64encode(
-                    self.sasl_username.encode('utf-8')).decode('utf-8')
-            elif self.sasl == 'plain':
-                authstring = base64.b64encode('\0'.join([
-                    self.sasl_username,
-                    self.sasl_username,
-                    self.sasl_password
-                ]).encode('utf-8')).decode('utf-8')
-
-            self.queueMsg(ircmsgs.IrcMsg(command='AUTHENTICATE', args=(authstring,)))
-        elif (len(msg.args) == 1 and msg.args[0] != '+' and
-                self.sasl == 'ecdsa-nist256p-challenge'):
+        mechanism = self.sasl_current_mechanism
+        if mechanism == 'ecdsa-nist256p-challenge':
+            if string == b'':
+                self.sendSaslString(self.sasl_username.encode('utf-8'))
+                return
             try:
-                private_key = SigningKey.from_pem(open(self.sasl_ecdsa_key).
-                    read())
-                authstring = base64.b64encode(
-                    private_key.sign(base64.b64decode(msg.args[0].encode()))).decode('utf-8')
-            except (BadDigestError, OSError, ValueError) as e:
-                authstring = "*"
-
-            self.queueMsg(ircmsgs.IrcMsg(command='AUTHENTICATE', args=(authstring,)))
-
-    def doCap(self, msg):
-        if len(msg.args) == 3:
-            for cap in msg.args[2].split(' '):
-                if msg.args[1] == 'ACK':
-                    log.info('%s: Server acknowledged capability %r',
-                             self.network, cap)
-
-                    if cap == 'sasl':
-                        self.queueMsg(ircmsgs.IrcMsg(
-                            command='AUTHENTICATE',
-                            args=(self.sasl.upper(),)))
-                elif msg.args[1] == 'NAK':
-                    log.warning('%s: Server refused capability %r',
-                                self.network, cap)
-
-                    if cap == 'sasl':
-                        self.queueMsg(ircmsgs.IrcMsg(
-                            command='CAP',
-                            args=('END',)))
+                with open(self.sasl_ecdsa_key) as fd:
+                    private_key = SigningKey.from_pem(fd.read())
+                authstring = private_key.sign(base64.b64decode(msg.args[0].encode()))
+                self.sendSaslString(authstring)
+            except (BadDigestError, OSError, ValueError):
+                self.sendMsg(ircmsgs.IrcMsg(command='AUTHENTICATE',
+                    args=('*',)))
+                self.tryNextSaslMechanism()
+        elif mechanism == 'external':
+            self.sendSaslString(b'')
+        elif mechanism == 'plain':
+            authstring = b'\0'.join([
+                self.sasl_username.encode('utf-8'),
+                self.sasl_username.encode('utf-8'),
+                self.sasl_password.encode('utf-8'),
+            ])
+            self.sendSaslString(authstring)
 
     def do903(self, msg):
         log.info('%s: SASL authentication successful', self.network)
-        self.queueMsg(ircmsgs.IrcMsg(command='CAP', args=('END',)))
+        self.sasl_authenticated = True
+        self.endCapabilityNegociation()
 
     def do904(self, msg):
-        if (self.sasl != 'plain' and self.sasl_username and
-                self.sasl_password):
-            log.info('%s: SASL %s failed, trying PLAIN.', self.network,
-                self.sasl.upper())
-
-            self.sasl = 'plain'
-
-            self.queueMsg(ircmsgs.IrcMsg(
-                command='AUTHENTICATE', args=(self.sasl.upper(),)))
-        else:
-            log.warning('%s: SASL authentication failed', self.network)
-            self.queueMsg(ircmsgs.IrcMsg(command='CAP', args=('END',)))
+        log.warning('%s: SASL authentication failed', self.network)
+        self.tryNextSaslMechanism()
 
     def do905(self, msg):
         log.warning('%s: SASL authentication failed because the username or '
                     'password is too long.', self.network)
-        self.queueMsg(ircmsgs.IrcMsg(command='CAP', args=('END',)))
+        self.tryNextSaslMechanism()
 
     def do906(self, msg):
         log.warning('%s: SASL authentication aborted', self.network)
-        self.queueMsg(ircmsgs.IrcMsg(command='CAP', args=('END',)))
+        self.tryNextSaslMechanism()
 
     def do907(self, msg):
         log.warning('%s: Attempted SASL authentication when we were already '
                     'authenticated.', self.network)
-        self.queueMsg(ircmsgs.IrcMsg(command='CAP', args=('END',)))
+        self.tryNextSaslMechanism()
 
     def do908(self, msg):
         log.info('%s: Supported SASL mechanisms: %s',
                  self.network, msg.args[1])
+        self.filterSaslMechanisms(set(msg.args[1].split(',')))
+
+    def doCap(self, msg):
+        subcommand = msg.args[1]
+        if subcommand == 'ACK':
+            self.doCapAck(msg)
+        elif subcommand == 'NAK':
+            self.doCapNak(msg)
+        elif subcommand == 'LS':
+            self.doCapLs(msg)
+        elif subcommand == 'DEL':
+            self.doCapDel(msg)
+        elif subcommand == 'NEW':
+            self.doCapNew(msg)
+    def doCapAck(self, msg):
+        if len(msg.args) != 3:
+            log.warning('Bad CAP ACK from server: %r', msg)
+            return
+        caps = msg.args[2].split()
+        assert caps, 'Empty list of capabilities'
+        log.debug('%s: Server acknowledged capabilities: %L',
+                 self.network, caps)
+        self.state.capabilities_ack.update(caps)
+
+        if 'sasl' in caps:
+            self.tryNextSaslMechanism()
+        else:
+            self.endCapabilityNegociation()
+    def doCapNak(self, msg):
+        if len(msg.args) != 3:
+            log.warning('Bad CAP NAK from server: %r', msg)
+            return
+        caps = msg.args[2].split()
+        assert caps, 'Empty list of capabilities'
+        self.state.capabilities_nak.update(caps)
+        log.warning('%s: Server refused capabilities: %L',
+                    self.network, caps)
+        self.endCapabilityNegociation()
+    def _addCapabilities(self, capstring):
+        for item in capstring.split():
+            while item.startswith(('=', '~')):
+                item = item[1:]
+            if '=' in item:
+                (cap, value) = item.split('=', 1)
+                self.state.capabilities_ls[cap] = value
+            else:
+                self.state.capabilities_ls[item] = None
+    def doCapLs(self, msg):
+        if len(msg.args) == 4:
+            # Multi-line LS
+            if msg.args[2] != '*':
+                log.warning('Bad CAP LS from server: %r', msg)
+                return
+            self._addCapabilities(msg.args[3])
+        elif len(msg.args) == 3: # End of LS
+            self._addCapabilities(msg.args[2])
+            common_supported_capabilities = set(self.state.capabilities_ls) & \
+                    self.REQUEST_CAPABILITIES
+            if 'sasl' in self.state.capabilities_ls:
+                s = self.state.capabilities_ls['sasl']
+                if s is not None:
+                    self.filterSaslMechanisms(set(s.split(',')))
+            if 'starttls' not in self.state.capabilities_ls and \
+                    self.requireStarttls:
+                log.error('%s: Server does not support STARTTLS.', self.network)
+                self.feedMsg(ircmsgs.error('STARTTLS upgrade not supported '
+                    'by the server'))
+                self.die()
+                return
+            # NOTE: Capabilities are requested in alphabetic order, because
+            # sets are unordered, and their "order" is nondeterministic.
+            # This is needed for the tests.
+            if common_supported_capabilities:
+                caps = ' '.join(sorted(common_supported_capabilities))
+                self.sendMsg(ircmsgs.IrcMsg(command='CAP',
+                    args=('REQ', caps)))
+            else:
+                self.endCapabilityNegociation()
+        else:
+            log.warning('Bad CAP LS from server: %r', msg)
+            return
+    def doCapDel(self, msg):
+        if len(msg.args) != 3:
+            log.warning('Bad CAP DEL from server: %r', msg)
+            return
+        caps = msg.args[2].split()
+        assert caps, 'Empty list of capabilities'
+        for cap in caps:
+            # The spec says "If capability negotiation 3.2 was used, extensions
+            # listed MAY contain values." for CAP NEW and CAP DEL
+            cap = cap.split('=')[0]
+            try:
+                del self.state.capabilities_ls[cap]
+            except KeyError:
+                pass
+            try:
+                self.state.capabilities_ack.remove(cap)
+            except KeyError:
+                pass
+    def doCapNew(self, msg):
+        if len(msg.args) != 3:
+            log.warning('Bad CAP NEW from server: %r', msg)
+            return
+        caps = msg.args[2].split()
+        assert caps, 'Empty list of capabilities'
+        self._addCapabilities(msg.args[2])
+        if not self.sasl_authenticated and 'sasl' in self.state.capabilities_ls:
+            self.resetSasl()
+            s = self.state.capabilities_ls['sasl']
+            if s is not None:
+                self.filterSaslMechanisms(set(s.split(',')))
+        common_supported_unrequested_capabilities = (
+                set(self.state.capabilities_ls) &
+                self.REQUEST_CAPABILITIES -
+                self.state.capabilities_ack)
+        if common_supported_unrequested_capabilities:
+            caps = ' '.join(sorted(common_supported_unrequested_capabilities))
+            self.sendMsg(ircmsgs.IrcMsg(command='CAP',
+                args=('REQ', caps)))
+
+    def monitor(self, targets):
+        """Increment a counter of how many callbacks monitor each target;
+        and send a MONITOR + to the server if the target is not yet
+        monitored."""
+        if isinstance(targets, str):
+            targets = [targets]
+        not_yet_monitored = set()
+        for target in targets:
+            if target in self.monitoring:
+                self.monitoring[target] += 1
+            else:
+                not_yet_monitored.add(target)
+                self.monitoring[target] = 1
+        if not_yet_monitored:
+            self.queueMsg(ircmsgs.monitor('+', not_yet_monitored))
+        return not_yet_monitored
+
+    def unmonitor(self, targets):
+        """Decrements a counter of how many callbacks monitor each target;
+        and send a MONITOR - to the server if the counter drops to 0."""
+        if isinstance(targets, str):
+            targets = [targets]
+        should_be_unmonitored = set()
+        for target in targets:
+            self.monitoring[target] -= 1
+            if self.monitoring[target] == 0:
+                del self.monitoring[target]
+                should_be_unmonitored.add(target)
+        if should_be_unmonitored:
+            self.queueMsg(ircmsgs.monitor('-', should_be_unmonitored))
+        return should_be_unmonitored
 
     def _getNextNick(self):
         if self.alternateNicks:
@@ -1129,8 +1358,11 @@ class Irc(IrcCommandDispatcher):
         if not self.afterConnect:
             newNick = self._getNextNick()
             assert newNick != self.nick
-            log.info('Got 433: %s %s.  Trying %s.',self.nick, problem, newNick)
+            log.info('Got %s: %s %s.  Trying %s.',
+                     msg.command, self.nick, problem, newNick)
             self.sendMsg(ircmsgs.nick(newNick))
+    def do437(self, msg):
+        self.do43x(msg, 'is temporarily unavailable')
     def do433(self, msg):
         self.do43x(msg, 'is in use')
     def do432(self, msg):
@@ -1158,7 +1390,7 @@ class Irc(IrcCommandDispatcher):
         """Handles ERROR messages."""
         log.warning('Error message from %s: %s', self.network, msg.args[0])
         if not self.zombie:
-           if msg.args[0].startswith('Closing Link'):
+           if msg.args[0].lower().startswith('closing link'):
               self.driver.reconnect()
            elif 'too fast' in msg.args[0]: # Connecting too fast.
               self.driver.reconnect(wait=True)
