@@ -35,12 +35,27 @@ import sys
 import time
 import shutil
 import urllib
-import httplib
 import unittest
+import functools
 import threading
 
 from . import (callbacks, conf, drivers, httpserver, i18n, ircdb, irclib,
         ircmsgs, ircutils, log, plugin, registry, utils, world)
+from .utils import minisix
+
+if minisix.PY2:
+    from httplib import HTTPConnection
+    from urllib import splithost, splituser
+    from urllib import URLopener
+else:
+    from http.client import HTTPConnection
+    from urllib.parse import splithost, splituser
+    from urllib.request import URLopener
+
+class verbosity:
+    NONE = 0
+    EXCEPTIONS = 1
+    MESSAGES = 2
 
 i18n.import_conf()
 network = True
@@ -58,6 +73,28 @@ def cachingGetHelp(method, name=None, doc=None):
     return lastGetHelp
 callbacks.getHelp = cachingGetHelp
 
+def retry(tries=3):
+    assert tries > 0
+    def decorator(f):
+        @functools.wraps(f)
+        def newf(self):
+            try:
+                f(self)
+            except AssertionError as e:
+                first_exception = e
+            for _ in range(1, tries):
+                try:
+                    f(self)
+                except AssertionError as e:
+                    pass
+                else:
+                    break
+            else:
+                # All failed
+                raise first_exception
+        return newf
+    return decorator
+
 def getTestIrc():
     irc = irclib.Irc('test')
     # Gotta clear the connect messages (USER, NICK, etc.)
@@ -194,9 +231,9 @@ class PluginTestCase(SupyTestCase):
         MiscModule = plugin.loadPluginModule('Misc')
         OwnerModule = plugin.loadPluginModule('Owner')
         ConfigModule = plugin.loadPluginModule('Config')
-        _ = plugin.loadPluginClass(self.irc, MiscModule)
-        _ = plugin.loadPluginClass(self.irc, OwnerModule)
-        _ = plugin.loadPluginClass(self.irc, ConfigModule)
+        plugin.loadPluginClass(self.irc, MiscModule)
+        plugin.loadPluginClass(self.irc, OwnerModule)
+        plugin.loadPluginClass(self.irc, ConfigModule)
         if isinstance(self.plugins, str):
             self.plugins = [self.plugins]
         else:
@@ -204,9 +241,9 @@ class PluginTestCase(SupyTestCase):
                 if name not in ('Owner', 'Misc', 'Config'):
                     module = plugin.loadPluginModule(name,
                                                      ignoreDeprecation=True)
-                    cb = plugin.loadPluginClass(self.irc, module)
+                    plugin.loadPluginClass(self.irc, module)
         self.irc.addCallback(TestInstance)
-        for (name, value) in self.config.iteritems():
+        for (name, value) in self.config.items():
             group = conf.supybot
             parts = registry.split(name)
             if parts[0] == 'supybot':
@@ -220,7 +257,7 @@ class PluginTestCase(SupyTestCase):
         if self.__class__ in (PluginTestCase, ChannelPluginTestCase):
             # Necessary because there's a test in here that shouldn\'t run.
             return
-        for (group, original) in self.originals.iteritems():
+        for (group, original) in self.originals.items():
             group.setValue(original)
         ircdb.users.close()
         ircdb.ignores.close()
@@ -230,23 +267,25 @@ class PluginTestCase(SupyTestCase):
         gc.collect()
 
     def _feedMsg(self, query, timeout=None, to=None, frm=None,
-                 usePrefixChar=True):
+                 usePrefixChar=True, expectException=False):
         if to is None:
             to = self.irc.nick
         if frm is None:
             frm = self.prefix
         if timeout is None:
             timeout = self.timeout
-        if self.myVerbose:
+        if self.myVerbose >= verbosity.MESSAGES:
             print('') # Extra newline, so it's pretty.
         prefixChars = conf.supybot.reply.whenAddressedBy.chars()
         if not usePrefixChar and query[0] in prefixChars:
             query = query[1:]
-        if sys.version_info[0] < 3:
+        if minisix.PY2:
             query = query.encode('utf8') # unicode->str
         msg = ircmsgs.privmsg(to, query, prefix=frm)
-        if self.myVerbose:
+        if self.myVerbose >= verbosity.MESSAGES:
             print('Feeding: %r' % msg)
+        if not expectException and self.myVerbose >= verbosity.EXCEPTIONS:
+            conf.supybot.log.stdout.setValue(True)
         self.irc.feedMsg(msg)
         fed = time.time()
         response = self.irc.takeMsg()
@@ -254,8 +293,10 @@ class PluginTestCase(SupyTestCase):
             time.sleep(0.1) # So it doesn't suck up 100% cpu.
             drivers.run()
             response = self.irc.takeMsg()
-        if self.myVerbose:
+        if self.myVerbose >= verbosity.MESSAGES:
             print('Response: %r' % response)
+        if not expectException and self.myVerbose >= verbosity.EXCEPTIONS:
+            conf.supybot.log.stdout.setValue(False)
         return response
 
     def getMsg(self, query, **kwargs):
@@ -274,7 +315,7 @@ class PluginTestCase(SupyTestCase):
     # But that would be hard, so I don't bother.  When this breaks, it'll get
     # fixed, but not until then.
     def assertError(self, query, **kwargs):
-        m = self._feedMsg(query, **kwargs)
+        m = self._feedMsg(query, expectException=True, **kwargs)
         if m is None:
             raise TimeoutError(query)
         if lastGetHelp not in m.args[1]:
@@ -422,7 +463,7 @@ class ChannelPluginTestCase(PluginTestCase):
         self.assertEqual(m.command, 'WHO')
 
     def _feedMsg(self, query, timeout=None, to=None, frm=None, private=False,
-                 usePrefixChar=True):
+                 usePrefixChar=True, expectException=False):
         if to is None:
             if private:
                 to = self.irc.nick
@@ -432,15 +473,17 @@ class ChannelPluginTestCase(PluginTestCase):
             frm = self.prefix
         if timeout is None:
             timeout = self.timeout
-        if self.myVerbose:
+        if self.myVerbose >= verbosity.MESSAGES:
             print('') # Newline, just like PluginTestCase.
         prefixChars = conf.supybot.reply.whenAddressedBy.chars()
         if query[0] not in prefixChars and usePrefixChar:
             query = prefixChars[0] + query
-        if sys.version_info[0] < 3 and isinstance(query, unicode):
+        if minisix.PY2 and isinstance(query, unicode):
             query = query.encode('utf8') # unicode->str
+        if not expectException and self.myVerbose >= verbosity.EXCEPTIONS:
+            conf.supybot.log.stdout.setValue(True)
         msg = ircmsgs.privmsg(to, query, prefix=frm)
-        if self.myVerbose:
+        if self.myVerbose >= verbosity.MESSAGES:
             print('Feeding: %r' % msg)
         self.irc.feedMsg(msg)
         fed = time.time()
@@ -465,8 +508,10 @@ class ChannelPluginTestCase(PluginTestCase):
                 ret = response
         else:
             ret = None
-        if self.myVerbose:
+        if self.myVerbose >= verbosity.MESSAGES:
             print('Returning: %r' % ret)
+        if not expectException and self.myVerbose >= verbosity.EXCEPTIONS:
+            conf.supybot.log.stdout.setValue(False)
         return ret
 
     def feedMsg(self, query, to=None, frm=None, private=False):
@@ -511,15 +556,15 @@ def open_http(url, data=None):
     user_passwd = None
     proxy_passwd= None
     if isinstance(url, str):
-        host, selector = urllib.splithost(url)
+        host, selector = splithost(url)
         if host:
-            user_passwd, host = urllib.splituser(host)
+            user_passwd, host = splituser(host)
             host = urllib.unquote(host)
         realhost = host
     else:
         host, selector = url
         # check whether the proxy contains authorization information
-        proxy_passwd, host = urllib.splituser(host)
+        proxy_passwd, host = splituser(host)
         # now we proceed with the url we want to obtain
         urltype, rest = urllib.splittype(selector)
         url = rest
@@ -527,9 +572,9 @@ def open_http(url, data=None):
         if urltype.lower() != 'http':
             realhost = None
         else:
-            realhost, rest = urllib.splithost(rest)
+            realhost, rest = splithost(rest)
             if realhost:
-                user_passwd, realhost = urllib.splituser(realhost)
+                user_passwd, realhost = splituser(realhost)
             if user_passwd:
                 selector = "%s://%s%s" % (urltype, realhost, rest)
             if urllib.proxy_bypass(realhost):
@@ -559,18 +604,20 @@ def open_http(url, data=None):
     if proxy_auth: c.putheader('Proxy-Authorization', 'Basic %s' % proxy_auth)
     if auth: c.putheader('Authorization', 'Basic %s' % auth)
     if realhost: c.putheader('Host', realhost)
-    for args in urllib.URLopener().addheaders: c.putheader(*args)
+    for args in URLopener().addheaders: c.putheader(*args)
     c.endheaders()
     return c
 
-class FakeHTTPConnection(httplib.HTTPConnection):
+class FakeHTTPConnection(HTTPConnection):
     _data = ''
     _headers = {}
     def __init__(self, rfile, wfile):
-        httplib.HTTPConnection.__init__(self, 'localhost')
+        HTTPConnection.__init__(self, 'localhost')
         self.rfile = rfile
         self.wfile = wfile
     def send(self, data):
+        if minisix.PY3 and isinstance(data, bytes):
+            data = data.decode()
         self.wfile.write(data)
     #def putheader(self, name, value):
     #    self._headers[name] = value
@@ -585,12 +632,8 @@ class HTTPPluginTestCase(PluginTestCase):
 
     def request(self, url, method='GET', read=True, data={}):
         assert url.startswith('/')
-        try:
-            from io import BytesIO as StringIO
-        except ImportError:
-            from StringIO import StringIO
-        wfile = StringIO()
-        rfile = StringIO()
+        wfile = minisix.io.StringIO()
+        rfile = minisix.io.StringIO()
         connection = FakeHTTPConnection(wfile, rfile)
         connection.putrequest(method, url)
         connection.endheaders()
